from setuptools import setup

setup(
    name='st1mulol-fastapi',
    version='0.70.0',
    packages=['tests', 'tests.test_tutorial', 'tests.test_tutorial.test_body', 'tests.test_tutorial.test_cors',
              'tests.test_tutorial.test_wsgi', 'tests.test_tutorial.test_events', 'tests.test_tutorial.test_testing',
              'tests.test_tutorial.test_metadata', 'tests.test_tutorial.test_security',
              'tests.test_tutorial.test_settings', 'tests.test_tutorial.test_templates',
              'tests.test_tutorial.test_websockets', 'tests.test_tutorial.test_async_tests',
              'tests.test_tutorial.test_body_fields', 'tests.test_tutorial.test_first_steps',
              'tests.test_tutorial.test_path_params', 'tests.test_tutorial.test_body_updates',
              'tests.test_tutorial.test_dependencies', 'tests.test_tutorial.test_extra_models',
              'tests.test_tutorial.test_query_params', 'tests.test_tutorial.test_cookie_params',
              'tests.test_tutorial.test_header_params', 'tests.test_tutorial.test_request_files',
              'tests.test_tutorial.test_request_forms', 'tests.test_tutorial.test_sql_databases',
              'tests.test_tutorial.test_behind_a_proxy', 'tests.test_tutorial.test_response_model',
              'tests.test_tutorial.test_custom_response', 'tests.test_tutorial.test_handling_errors',
              'tests.test_tutorial.test_background_tasks', 'tests.test_tutorial.test_extra_data_types',
              'tests.test_tutorial.test_response_cookies', 'tests.test_tutorial.test_response_headers',
              'tests.test_tutorial.test_sub_applications', 'tests.test_tutorial.test_extending_openapi',
              'tests.test_tutorial.test_openapi_callbacks', 'tests.test_tutorial.test_body_nested_models',
              'tests.test_tutorial.test_advanced_middleware', 'tests.test_tutorial.test_async_sql_databases',
              'tests.test_tutorial.test_bigger_applications', 'tests.test_tutorial.test_conditional_openapi',
              'tests.test_tutorial.test_additional_responses', 'tests.test_tutorial.test_body_multiple_params',
              'tests.test_tutorial.test_schema_extra_example', 'tests.test_tutorial.test_sql_databases_peewee',
              'tests.test_tutorial.test_testing_dependencies', 'tests.test_tutorial.test_additional_status_codes',
              'tests.test_tutorial.test_request_forms_and_files', 'tests.test_tutorial.test_custom_request_and_route',
              'tests.test_tutorial.test_response_change_status_code',
              'tests.test_tutorial.test_query_params_str_validations',
              'tests.test_tutorial.test_path_operation_configurations',
              'tests.test_tutorial.test_path_operation_advanced_configurations', 'tests.test_modules_same_name_body',
              'tests.test_modules_same_name_body.app', 'fastapi', 'fastapi.openapi', 'fastapi.security',
              'fastapi.middleware', 'fastapi.dependencies'],
    url='https://github.com/st1mulol/fastapi',
    license='MIT license',
    author='St1muLoL',
    author_email='st1mul1337.trap@gmail.com',
    description='При получении параметров POST типа "arg=" значение заменяется на пустую строку вместо None'
)
